@@ -58,7 +58,7 @@ setMethodS3("getChecksumObjects", "GenericDataFileSet", function(this, ..., verb
   verbose && enter(verbose, "Retrieveing checksums for all files");
   fullnames <- getFullNames(this);
 
-  checksums <- character(length(n));
+  checksums <- character(length=n)
   for (ii in seq_len(n)) {
     checksums[ii] <- getChecksum(this[[ii]]);
     verbose && printf(verbose, "File #%d ('%s') checksum: %s\n", ii, fullnames[ii], checksums[ii]);
