@@ -15,18 +15,18 @@ print(ds)
 # Read column 'y' and a subset of the rows from each of the
 # tab-delimited files and combine into a matrix
 rows <- c(3:5, 8, 2)
-data <- extractMatrix(ds, column="y", colClass="integer", rows=rows)
+data <- extractMatrix(ds, column="y", colClass="integer", rows=rows, drop=TRUE)
 print(data)
 
 
 # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 # Read data frames from each of the files
 # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-dataList <- lapply(ds, readDataFrame)
+dataList <- lapply(ds, FUN=readDataFrame)
 print(dataList)
 
 rows <- c(3:5, 8, 2)
-dataList <- lapply(ds, readDataFrame, rows=rows)
+dataList <- lapply(ds, FUN=readDataFrame, rows=rows)
 print(dataList)
 
 
@@ -44,7 +44,7 @@ names(colClasses) <- colNames
 cat("Column class patterns:\n")
 print(colClasses)
 
-data <- readDataFrame(ds, colClasses=colClasses)
+data <- readDataFrame(ds, colClasses=colClasses, verbose=TRUE)
 print(data)
 
 
